require_relative '../spec_helper'
require_relative '../../lib/sastbox-sdk'

RSpec.describe 'Scanner' do
  let(:scanner) do
    SastBox::Scanner.new(
      name: 'test_name',
      name_alias: 'test_alias',
      description: 'test_desc',
      support: ['test_support'],
      version: '0.1',
      tool: 'test_tool'
    )
  end

  describe 'start_scan' do
    context 'should raise name error for run method when correct options are provided' do
      before do
        scanner.parse_opts(['-o', 'xxx', '-c', 'aaa'])
      end

      it "should raise name error for run method when correct options are provided" do
        expect { scanner.start_scan }.to raise_error(NameError) do |error| # #<NameError: undefined local variable or method `run' ...
          expect( error.message ).to match(/undefined.*run/)
        end
      end
    end
  end

  describe 'finish_scan' do
    context 'should exit with status 1' do
      let(:filename) do
        File.absolute_path(File.join(File.dirname(__FILE__), '..', 'samples/low.php'))
      end

      let(:snippet) do
        scanner.snippet_read(filename, 10)
      end

      let(:issue) do
        {
          title: 'vuln_name',
          description: 'vuln_description',
          references: [],
          filename: filename,
          snippet: snippet
        }
      end

      before do
        scanner.parse_opts(['-o', 'xxx', '-c', 'aaa'])
        scanner.add_issue(issue)
      end

      it do
        expect { scanner.finish_scan }.to raise_error(SystemExit) do |error|
          expect( error.status ).to eq(1)
        end
      end
    end

    context 'should exit with status 0' do
      let(:filename) do
        File.absolute_path(File.join(File.dirname(__FILE__), '..', 'samples/low.php'))
      end

      let(:snippet) do
        scanner.snippet_read(filename, 10)
      end

      before do
        scanner.parse_opts(['-o', 'xxx', '-c', 'aaa'])
      end

      it do
        expect { scanner.finish_scan }.to raise_error(SystemExit) do |error|
          expect( error.status ).to eq(0)
        end
      end
    end
  end

  describe 'info' do
    subject { scanner.info }
    it 'name' do
      expect(subject[:name]).to eq 'test_name'
    end

    it 'description' do
      expect(subject[:description]).to eq 'test_desc'
    end

    it 'support' do
      expect(subject[:support]).to eq ['test_support']
    end

    it 'version' do
      expect(subject[:version]).to eq '0.1'
    end
  end

  describe 'add_issue' do
    context 'when issue should be skipped' do

      let(:snippet) do
        {evidence_line: {content: 'abc', hash: ''}, evidence_full: {content: 'abc', hash: ''}}
      end

      let(:issue) do
        {
          title: 'vuln_name',
          description: 'vuln_description',
          references: [],
          filename: 'base/.git/xxx',
          snippet: snippet
        }
      end

      before do
        @status = scanner.add_issue(issue)
      end

      it { expect(@status).to be nil }
      it { expect(scanner.issues.length).to be 0}
    end

    context 'when issue should not be skipped' do

      let(:filename) do
        File.absolute_path(File.join(File.dirname(__FILE__), '..', 'samples/low.php'))
      end

      let(:snippet) do
        scanner.snippet_read(filename, 10)
      end

      let(:issue) do
        {
          title: 'vuln_name',
          description: 'vuln_description',
          references: [],
          filename: filename,
          snippet: snippet
        }
      end

      before do
        scanner.parse_opts(['-c', '/abc/def/ghi'])
        @status = scanner.add_issue(issue)
      end

      #it { expect(@status).to be scanner.issues }
      it { expect(scanner.issues.length).to be 1}
      it "hash_issue_v1" do
        expect(scanner.issues.first[:hash_issue]).to eq '220da2ef86cb34b97e4253f5b212b2a720c93d717e7fec8f243d851441774181'
      end

      it "hash_issue_v2" do
        expect(scanner.issues.first[:hash_issue_v2]).to eq '55a52215b49450ebf8fdb8db1d50f4be3d204959989834e25ba2be9e2f2d5b00'
      end

    end
  end

  describe 'parse_json_from_str' do
    context 'should parse json from string' do
      it "should parse json from string" do
        expect(scanner.parse_json_from_str('{"abc": "123"}') ).to be_an Hash
        expect(scanner.parse_json_from_str('{"abc": "123"}') ).to have_key("abc")
        expect(scanner.parse_json_from_str('{"abc": "123"}')["abc"] ).to eq "123"
      end

      it "should return nil when json is invalid" do
        expect(scanner.parse_json_from_str('xxxxx') ).to eq nil
      end
    end
  end

  describe 'parse_json_from_file' do
    context 'should parse json from file' do
      let(:filename_valid) { '/tmp/json_input_valid_test.json' }
      let(:filename_invalid) { '/tmp/json_input_invalid_test.json' }

      before do
        File.write(filename_valid, '{"abc": "123"}')
        File.write(filename_invalid, 'xxxxx')
      end

      it "should parse json from file" do
        expect(scanner.parse_json_from_file(filename_valid) ).to be_an Hash
        expect(scanner.parse_json_from_file(filename_valid) ).to have_key("abc")
        expect(scanner.parse_json_from_file(filename_valid)["abc"] ).to eq "123"
      end

      it "should return nil when json is invalid" do
        expect(scanner.parse_json_from_file(filename_invalid) ).to eq nil
      end
    end
  end

  describe 'save_results' do
    context 'should save results to file' do
      let(:report) { '/tmp/save_results.json' }

      before do
        File.delete(report) if File.exist?(report)

        allow(scanner).to receive(:generate_sarif_report) { '{}' }

        scanner.parse_opts(['-o', report])
        scanner.save_scan_output
      end

      it "should save results to file" do
        expect(File.exist?(report)).to be true
      end
    end
  end

  describe 'gen_random_tmp_filename' do
    context 'should generate random filename' do

      it "should generate random filename" do
        expect(scanner.gen_random_tmp_filename('.xxxyyy')).to match /\/tmp\/.*\.xxxyyy/
      end
    end
  end

  describe 'gen_random_tmp_filename_json' do
    context 'should generate random json filename' do

      it "should generate random json filename" do
        expect(scanner.gen_random_tmp_filename_json).to match /\/tmp\/.*\.json/
      end
    end
  end
end

