require_relative '../spec_helper'
require_relative '../../lib/sastbox-sdk'


RSpec.describe 'snippet' do
  let(:scanner) do
    SastBox::Scanner.new(
      name: 'test_name',
      name_alias: 'test_alias',
      description: 'test_desc',
      support: ['test_support'],
      version: '0.1',
      tool: 'test_tool'
    )
  end

  let(:filename) do
    File.absolute_path(File.join(File.dirname(__FILE__), '..', 'samples/low.php'))
  end

  describe 'filename_relative' do
    before do
      scanner.parse_opts(['-c', '/abc/def/ghi'])
    end

    context 'filename does not starts with codebase' do
      it {expect( scanner.filename_relative('/xxx/yyy') ).to be nil }
    end

    context 'filename starts with codebase' do
      it {expect( scanner.filename_relative('/abc/def/ghi/xxx/yyy') ).to eq 'xxx/yyy' }
    end
  end

  describe 'snippet_calculate_hashes' do
    context 'hashes to match correct value' do
      let(:snippet) do
        {evidence_line: {content: 'abc', hash: ''}, evidence_full: {content: 'abc', hash: ''}}
      end

      before do
        scanner.snippet_calculate_hashes(snippet)
      end

      it {expect( snippet[:evidence_line][:hash]).to eq 'ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad' }

      it {expect( snippet[:evidence_full][:hash]).to eq 'ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad' }
    end
  end

  describe 'snippet_read' do
    context 'when file exists and line is within range' do
      subject {scanner.snippet_read(filename, 10) }

      it 'snippet[:read_success]' do
        expect(subject[:read_success]).to be true;
      end

      it 'snippet[:evidence_line][:content]' do
        expect(subject[:evidence_line][:content]).to eq "\t\t$cmd = shell_exec( 'ping  ' . $target );";
      end

      it 'snippet[:evidence_line][:hash]' do
        expect(subject[:evidence_line][:hash]).to eq '9882497a714594ae1d4ac2e167cce7de6cdc7dd4cffa5456bffd7b3194c9d042';
      end

      it 'snippet[:evidence_full][:hash]' do
        expect(subject[:evidence_full][:hash]).to eq 'c61da3e81b8847eb0b6b34bda42db8177e120b6b1fc8e19e0ed332d6751000f0';
      end
    end

    context 'when file does not exist' do
      subject {scanner.snippet_read('/tmp/qwuhasgnewhbeqwyegasdf', 10) }

      it 'snippet[:read_success]' do
        expect(subject[:read_success]).to be false;
      end

      it 'snippet[:evidence_line][:content]' do
        expect(subject[:evidence_line][:content]).to eq ''
      end

      it 'snippet[:evidence_line][:hash]' do
        expect(subject[:evidence_line][:hash]).to eq '';
      end

      it 'snippet[:evidence_full][:hash]' do
        expect(subject[:evidence_full][:hash]).to eq '';
      end
    end

    context 'when file exists and line is out of range' do
      subject {scanner.snippet_read(filename, 50000) }

      it 'snippet[:read_success]' do
        expect(subject[:read_success]).to be false;
      end

      it 'snippet[:evidence_line][:content]' do
        expect(subject[:evidence_line][:content]).to eq ''
      end

      it 'snippet[:evidence_line][:hash]' do
        expect(subject[:evidence_line][:hash]).to eq '';
      end

      it 'snippet[:evidence_full][:hash]' do
        expect(subject[:evidence_full][:hash]).to eq '';
      end
    end
  end

  describe 'snippet_read_range' do
    context 'when file exists and start/end line is within range' do
      subject {scanner.snippet_read_range(filename, 17, 18) }

      it 'snippet[:read_success]' do
        expect(subject[:read_success]).to be true;
      end

      it 'snippet[:evidence_line][:content]' do
        expect(subject[:evidence_line][:content]).to eq "\t// Feedback for the end user\r\n\t$html .= \"<pre>{$cmd}</pre>\";\r\n";
      end

      it 'snippet[:evidence_line][:hash]' do
        expect(subject[:evidence_line][:hash]).to eq '1beb122abba992b58df776523269522a6b0411679e991f62e2fa116db6d960ff';
      end

      it 'snippet[:evidence_full][:hash]' do
        expect(subject[:evidence_full][:hash]).to eq '9a4a2ee8ed7a70956e9960036e036770499f48225f20c42bb8d21a2f40771dc6';
      end
    end

    context 'when file does not exist' do
      subject {scanner.snippet_read_range('/tmp/qwuhasgnewhbeqwyegasdf', 17, 18) }

      it 'snippet[:read_success]' do
        expect(subject[:read_success]).to be false;
      end

      it 'snippet[:evidence_line][:content]' do
        expect(subject[:evidence_line][:content]).to eq ''
      end

      it 'snippet[:evidence_line][:hash]' do
        expect(subject[:evidence_line][:hash]).to eq '';
      end

      it 'snippet[:evidence_full][:hash]' do
        expect(subject[:evidence_full][:hash]).to eq '';
      end
    end

    context 'when file exists and start/end line is out of range' do
      subject {scanner.snippet_read_range(filename, 50000, 60000) }

      it 'snippet[:read_success]' do
        expect(subject[:read_success]).to be false;
      end

      it 'snippet[:evidence_line][:content]' do
        expect(subject[:evidence_line][:content]).to eq ''
      end

      it 'snippet[:evidence_line][:hash]' do
        expect(subject[:evidence_line][:hash]).to eq '';
      end

      it 'snippet[:evidence_full][:hash]' do
        expect(subject[:evidence_full][:hash]).to eq '';
      end
    end
  end
end

