require_relative '../spec_helper'
require_relative '../../lib/sastbox-sdk'


RSpec.describe 'Cwe_detector' do
  let(:scanner) do
    SastBox::Scanner.new(
    name: 'test_name',
    name_alias: 'test_alias',
    description: 'test_desc',
    support: ['test_support'],
    version: '0.1',
    tool: 'test_tool'
  )
  end

  describe 'cwe_found?' do
    context 'when issue has pattern' do
      let(:issue) do
      {
        title: 'cmd exec',
        description: 'cmd exec'
      }
      end

      let(:cwe_id) { 123 }

      before do
        scanner.alternative_titles(issue)
        @status = scanner.cwe_found?(issue, ['cmd exec'], cwe_id)
      end

      it { expect( issue[:cwe_id]).to eq cwe_id }
      it { expect( @status).to be true }
    end

    context 'when issue does not have pattern' do
      let(:issue) do
      {
        title: 'cmd exec',
        description: 'cmd exec'
      }
      end

      let(:cwe_id) { 123 }

      before do
        scanner.alternative_titles(issue)
      end

      it {expect( scanner.cwe_found?(issue, ['xxx'], cwe_id)).to be false}
      it {expect( issue[:cwe_id]).to eq nil}
    end
  end

  describe 'cwe_start_heuristics?' do
  end

  describe 'guess_cwe' do
    context 'when cwe_id is already assigned to valid CWE' do
      let(:issue) do
      {
        title: 'sql inj',
        description: 'sql inj',
        cwe_id: 123
      }
      end

      before do
        scanner.guess_cwe(issue)
      end

      it {expect(issue[:cwe_id]).to be 123}
    end

    context 'when cwe_id is already assigned to invalid CWE' do
      let(:issue) do
      {
        title: 'sql inj',
        description: 'sql inj',
        cwe_id: -1
      }
      end

      before do
        scanner.guess_cwe(issue)
      end

      it {expect(issue[:cwe_id]).to eq SastBox::Cwe::SQL_INJECTION}
    end

    ############################################################################
    context 'when has sql injection pattern and cwe_id not set' do
      let(:issue) { { title: 'sql inj', description: 'sql inj' } }
      before { scanner.guess_cwe(issue) }
      it 'sql injection' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::SQL_INJECTION
      end
    end
    ############################################################################
    context 'when has xss pattern and cwe_id not set' do
      let(:issue) { { title: 'xss', description: 'xss' } }
      before { scanner.guess_cwe(issue) }
      it 'xss' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::XSS
      end
    end
    ############################################################################
    context 'when has cmd injection pattern and cwe_id not set' do
      let(:issue) { { title: 'cmd injection', description: 'cmd injection' } }
      before { scanner.guess_cwe(issue) }
      it 'cmd injection' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::OS_COMMAND_INJECTION
      end
    end
    ############################################################################
    context 'when has code injection pattern and cwe_id not set' do
      let(:issue) { { title: 'code injection', description: 'code injection' } }
      before { scanner.guess_cwe(issue) }
      it 'code injection' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::CODE_INJECTION
      end
    end
    ############################################################################
    context 'when has session fixation pattern and cwe_id not set' do
      let(:issue) { { title: 'session fixation', description: 'session fixation' } }
      before { scanner.guess_cwe(issue) }
      it 'session fixation' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::SESSION_FIXATION
      end
    end
    ############################################################################
    context 'when has csrf pattern and cwe_id not set' do
      let(:issue) { { title: 'csrf', description: 'csrf' } }
      before { scanner.guess_cwe(issue) }
      it 'csrf' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::CSRF
      end
    end
    ############################################################################
    context 'when has deserialization pattern and cwe_id not set' do
      let(:issue) { { title: 'deserialization', description: 'deserialization' } }
      before { scanner.guess_cwe(issue) }
      it 'deserialization' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::DESERIALIZATION_OF_UNTRUSTED_DATA
      end
    end
    ############################################################################
    context 'when has path traversal pattern and cwe_id not set' do
      let(:issue) { { title: 'path traversal', description: 'path traversal' } }
      before { scanner.guess_cwe(issue) }
      it 'path traversal' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::PATH_TRAVERSAL
      end
    end
    ############################################################################
    context 'when has hardcoded password pattern and cwe_id not set' do
      let(:issue) { { title: 'hard-coded', description: 'hard-coded' } }
      before { scanner.guess_cwe(issue) }
      it 'hard-coded' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::HARD_CODED_PASSWORD
      end
    end
    ############################################################################
    context 'when has null pointer deref pattern and cwe_id not set' do
      let(:issue) { { title: 'null pointer deref', description: 'null pointer deref' } }
      before { scanner.guess_cwe(issue) }
      it 'null pointer deref' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::NULL_POINTER_DEREFERENCE
      end
    end
    ############################################################################
    context 'when has broken crypto pattern and cwe_id not set' do
      let(:issue) { { title: 'broken crypto', description: 'broken crypto' } }
      before { scanner.guess_cwe(issue) }
      it 'broken crypto' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::BROKEN_CRYPTO
      end
    end
    ############################################################################
    context 'when has improper authorization pattern and cwe_id not set' do
      let(:issue) { { title: 'improper authorization', description: 'improper authorization' } }
      before { scanner.guess_cwe(issue) }
      it 'improper authorization' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::IMPROPER_AUTHORIZATION
      end
    end
    ############################################################################
    context 'when has improper authentication pattern and cwe_id not set' do
      let(:issue) { { title: 'improper authentication', description: 'improper authentication' } }
      before { scanner.guess_cwe(issue) }
      it 'improper authentication' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::IMPROPER_AUTHENTICATION
      end
    end
    ############################################################################
    context 'when has improper input validation pattern and cwe_id not set' do
      let(:issue) { { title: 'input validation', description: 'input validation' } }
      before { scanner.guess_cwe(issue) }
      it 'input validation' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::IMPROPER_INPUT_VALIDATION
      end
    end
    ############################################################################
    context 'when has unrestricted file upload pattern and cwe_id not set' do
      let(:issue) { { title: 'unrestricted file upload', description: 'unrestricted file upload' } }
      before { scanner.guess_cwe(issue) }
      it 'unrestricted file upload' do
        expect(issue[:cwe_id]).to eq SastBox::Cwe::UNRESTRICTED_UPLOAD_OF_FILE_WITH_DANGEROUS_TYPE
      end
    end


  end

end
